import { Component, Input, OnInit } from '@angular/core';
import { Coordinates, Geolocation } from '@awesome-cordova-plugins/geolocation/ngx';
import { LayerGroup, Map, MapOptions, Marker, TileLayer, circle, icon, latLng } from 'leaflet';
import { NgxUiLoaderService } from 'ngx-ui-loader';
import { PlaceAddress } from 'src/app/models';
import { GlobalService, PoiService } from 'src/app/services';
import { MESSAGES } from 'src/app/utility';
import { DURATIONTOAST, OPEN_STREET_MAPS } from 'src/app/utility/global-constant';
//import { Geolocation } from '@awesome-cordova-plugins/geolocation/ngx';

@Component({
  selector: 'app-widget-mappa',
  templateUrl: './widget-mappa.component.html',
  styleUrls: ['./widget-mappa.component.scss'],
})
export class WidgetMappaComponent implements OnInit {
  @Input() placeAddress: PlaceAddress;
  map: Map;
  showMap: boolean = false;
  mapOptions: MapOptions;
  markers: Marker[] = [];
  markerMapLayerGroup = new LayerGroup();
  locationLayerGroup = new LayerGroup();
  baseMapLayerGroup = new LayerGroup();

  constructor(
    private poiService: PoiService,
    private geolocation: Geolocation,
    private globalService: GlobalService,
    private ngxService: NgxUiLoaderService,) { }

  ngOnInit() {
    this.setUtility();
  }

  setUtility() {
    this.initializeMapOptions();
    this.showMap = true;
  }

  getCoordsByAddress(placeAddress: PlaceAddress) {
    this.poiService.getCoordsByAddress(placeAddress).subscribe(
      {
        next: (result) => {
          if (result && result.length > 0) {
            this.placeAddress = {
              ...this.placeAddress, ...result[0]
            }
            this.placeAddress.lat && this.placeAddress.lon && this.addSingleMarker(this.placeAddress);
          }
        },
        error: (error) => {
        },
        complete: () => {
        },
      }
    );
  }

  onMapReady(map: Map) {
    this.map = map;
    this.map.addLayer(this.createBaseLayer());
    this.map.addLayer(this.markerMapLayerGroup);
    this.map.addLayer(this.locationLayerGroup);
    if (this.placeAddress) this.getCoordsByAddress(this.placeAddress);
  }

  createBaseLayer(): LayerGroup {
    this.baseMapLayerGroup.clearLayers();
    const baseMapTileLayer = new TileLayer('http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png');
    this.baseMapLayerGroup.addLayer(baseMapTileLayer);
    return this.baseMapLayerGroup;
  }

  addSingleMarker(placeAddress: PlaceAddress) {
    const marker = new Marker([Number(placeAddress.lat), Number(placeAddress.lon)])
      .setIcon(
        icon({
          iconSize: [25, 32],
          iconAnchor: [13, 32],
          iconUrl: 'assets/openmaps/markers_images/marker.png'
        }));
    marker.addTo(this.markerMapLayerGroup);
    this.setCenterMap(Number(this.placeAddress.lat), Number(this.placeAddress.lon));
  }

  initializeMapOptions() {
    this.mapOptions = {
      center: latLng(OPEN_STREET_MAPS.center.lat, OPEN_STREET_MAPS.center.lng),
      zoom: 16,
      maxZoom: 18,
      zoomControl: true,
      preferCanvas: true,
      attributionControl: true,
    };
  }

  setCenterMap(lat: number, lon: number) {
    this.map.setView(latLng(lat, lon), 16);
  }

  locate() {
    this.startAndStopLoader(true);
    this.geolocation.getCurrentPosition().then(
      (result) => {
        this.onLocationSuccess(result.coords);
      },
      () => {
        this.globalService.presentToast(MESSAGES.locationUnavailable, DURATIONTOAST.default);
        this.startAndStopLoader();
      },
    );
  }

  onLocationSuccess(coords: Coordinates) {
    this.locationLayerGroup.clearLayers();
    const { accuracy, latitude, longitude } = coords;
    const latlng = [latitude, longitude];
    this.setCenterMap(coords.latitude, coords.longitude);
    const locationCircle = circle(latlng as any, accuracy, {
      fillOpacity: 0.2,
      color: '#4285f4'
    });
    this.locationLayerGroup.getLayers().length == 0 && this.locationLayerGroup.addLayer(locationCircle)
    this.startAndStopLoader();
  }

  startAndStopLoader(start?: boolean) {
    start ? this.ngxService.start() : this.ngxService.stop();
  }

  /*   addMarkers(markers: Poi[]) {
      markers.forEach((poi) => this.addSingleMarker(poi));
      this.markerMapLayerGroup.addTo(this.map)
    } 
      removeMarkers() {
        this.clearLayers();
      }
        prepareMarkerRequest(category: string, boxView?: string): MarkerRequest {
      let markerRequest: MarkerRequest = {
        category: category
      };
      return boxView ? {
        ...markerRequest, ...{ boxView: boxView }
      } : markerRequest
    }
    */
}
